import React from "react";
import { Button, Layout } from "antd";
import "./App.css";
import axios from "axios";
import broker from "./Broker.js";
import { SearchString } from "./SearchString.js";
import { TermSelectTree } from "./TermSelectTree.js";
import { EditForm } from "./EditForm.js";
import { TermCard } from "./TermCard.js";

//Приложение

const { Header, Sider, Content } = Layout;

class App extends React.Component {
  state = {
    terms: []
  };

  constructor(props) {
    super(props);
    broker.onEvent("termSelected", data => this.onUpdateContent(0, data));
    broker.onEvent("searchClick", data => this.onUpdateContent(1, data));
  }

  onCreateTermClick = value => {
    //запускаем форму редактирования для создания нового термина
    broker.storeEvent("editTerm", null);
  };

  //поиск терминов по наименованию через поиск или отображение выбранного в дереве термина
  onUpdateContent(method, value) {
    let terms = [];
    let url = `/getTerm/${value}`; //выбор по id из дерева
    if (method === 1) {
      url = `/getTermsbySearchString/${value}`; //поиск по строке
    }
    axios.get(url).then(res => {
      terms = res.data;
      this.setState({ terms });
    });
  }

  //собираем все части вместе
  render() {
    return (
      <div>
        <Layout style={{ height: "100vh" }}>
          <Header>
            <table width="100%">
              <tbody>
                <tr>
                  <td>
                    <SearchString />
                  </td>
                  <td align="right">
                    <Button
                      type="small"
                      onClick={() => this.onCreateTermClick()}
                    >
                      Создать термин
                    </Button>
                  </td>
                </tr>
              </tbody>
            </table>
          </Header>
          <Layout>
            <Sider style={{ background: "#fff" }} width={300}>
              <TermSelectTree />
            </Sider>
            <Content>
              {this.state.terms &&
                this.state.terms.length > 0 &&
                this.state.terms.map(term => (
                  <TermCard term={term} key={term.id}></TermCard>
                ))}
            </Content>
          </Layout>
        </Layout>
        <EditForm />
      </div>
    );
  }
}

export default App;
