import React from "react";
import { Select, Modal, Form, Input, Icon, Button } from "antd";
import "./App.css";
import axios from "axios";
import broker from "./Broker.js";
import sources from "./Sources.js";

//Форма редактирование или создания термина

const { TextArea } = Input;
const { Option } = Select;
let id = 0;

class EditFormClass extends React.Component {
  state = {
    visible: false,
    term: {
      id: -1,
      key: -1,
      title: "",
      abbreviation: "",
      enName: "",
      extra: "",
      descriptions: [{ id: -1, text: "" }]
    }
  };

  constructor(props) {
    super(props);
    broker.onEvent("editTerm", data => this.onEditTerm(data));
  }

  onEditTerm(value) {
    //сбрасываем форму
    this.props.form.resetFields();
    //создание или редактирование термина?
    if (value && value.id) {
      this.setState({ visible: true, term: value });
    } else {
      this.setState({
        visible: true,
        term: {
          id: -1,
          key: -1,
          title: "",
          abbreviation: "",
          enName: "",
          descriptions: []
        }
      });
    }
  }

  handleSubmit = e => {
    //проверка и отправка значений формы
    e.preventDefault();
    this.props.form.validateFields((err, values) => {
      if (!err) {
        const { keys } = values;
        const descriptions = [];
        for (let k = 0; k < keys.length; k++) {
          descriptions.push({
            sourceId: values.descriptionSource[keys[k]],
            text: values.description[keys[k]]
          });
        }
        const objtosent = {
          id: this.state.term.id,
          name: values.name,
          abbreviation: values.abbreviation,
          enName: values.enName,
          descriptions: descriptions
        };
        axios.post(`/storeTerm`, objtosent).then(res => {
          const data = res.data;
          //закрываем форму
          this.setState({
            visible: false
          });
          //вновь показываем отредактированный или созданный термин
          broker.storeEvent("termSelected", data.id);
        });
      }
    });
  };

  handleDelete = e => {
    //удаляем термин
    new Promise(resolve => {
      axios.get(`/deleteTerm/${this.state.term.id}`).then(res => {
        resolve();
      });
    }).then(result => {
      //закрываем форму
      this.setState({
        visible: false
      });
      broker.storeEvent("termSelected", -1);
    });
  };

  handleCancel = e => {
    //просто закрываем форму
    this.setState({
      visible: false
    });
  };

  //удаляем описание термина
  remove = k => {
    const { form } = this.props;
    const keys = form.getFieldValue("keys");
    // Нам нужно хотябы одно определение
    if (keys.length === 1) {
      return;
    }
    form.setFieldsValue({
      keys: keys.filter(key => key !== k)
    });
  };

  //добавляем описание термина
  add = () => {
    const { form } = this.props;
    const keys = form.getFieldValue("keys");
    const nextKeys = keys.concat(id++);
    //уведомляем форму о изменениях
    form.setFieldsValue({
      keys: nextKeys
    });
  };

  //получаем описание термина по его id
  getDescriptionbyId = id => {
    let desc = { id: -1, text: "" };
    const { term } = this.state;
    if (term && term.id > -1) {
      term.descriptions.forEach(descr => {
        if (descr.id === id) {
          desc = descr;
        }
      });
    }
    return desc;
  };

  render() {
    const { term } = this.state;
    const { getFieldDecorator, getFieldValue } = this.props.form;
    const sourceOptions = sources.sources.map(source => (
      <Option value={source.id} key={source.id}>
        {sources.getSouceDatabyId(source.id)}
      </Option>
    ));

    const formItemLayoutWithOutLabel = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 6 }
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 18 }
      }
    };

    getFieldDecorator("keys", { initialValue: [-1] });
    if (term && term.id > -1) {
      const iniVal = [];
      term.descriptions.forEach(desc => {
        iniVal.push(desc.id);
      });
      getFieldDecorator("keys", { initialValue: iniVal });
    }
    const keys = getFieldValue("keys");

    const formItems = keys.map((k, index) => (
      <Form.Item
        {...formItemLayoutWithOutLabel}
        label={"Определение " + String(Number(index) + 1)}
        required={false}
        key={k}
      >
        {getFieldDecorator(`description[${k}]`, {
          validateTrigger: ["onChange", "onBlur"],
          initialValue: this.getDescriptionbyId(k).text,
          rules: [
            {
              required: true,
              whitespace: true,
              message: "Требуется ввести значение"
            }
          ]
        })(
          <TextArea
            rows={3}
            placeholder="Определение"
            style={{ width: "80%", marginRight: 8 }}
          />
        )}

        {getFieldDecorator(`descriptionSource[${k}]`, {
          initialValue:
            this.getDescriptionbyId(k).sourceId > -1
              ? this.getDescriptionbyId(k).sourceId
              : "",
          rules: []
        })(
          <Select style={{ width: "80%", marginRight: 8 }}>
            {sourceOptions}
          </Select>
        )}

        {keys.length > 1 ? (
          <Icon
            className="dynamic-delete-button"
            type="minus-circle-o"
            onClick={() => this.remove(k)}
          />
        ) : null}
      </Form.Item>
    ));

    let delButton;
    let formTitle = "Создание";
    if (term && term.id > -1) {
      formTitle = "Редактирование";
      delButton = (
        <Button type="small" onClick={this.handleDelete}>
          Удалить
        </Button>
      );
    }
    return (
      <Modal
        title={formTitle}
        visible={this.state.visible}
        onCancel={this.handleCancel}
        footer={[]}
        width={800}
      >
        <Form onSubmit={this.handleSubmit}>
          <Form.Item
            {...formItemLayoutWithOutLabel}
            label={"Термин: "}
            required={false}
            key="termId{term.id}"
          >
            {getFieldDecorator(`name`, {
              validateTrigger: ["onChange", "onBlur"],
              initialValue: term.name,
              rules: [
                {
                  required: true,
                  whitespace: true,
                  message: "Требуется ввести значение"
                }
              ]
            })(
              <Input
                placeholder="Наименование"
                style={{ width: "80%", marginRight: 8 }}
              />
            )}
          </Form.Item>

          <Form.Item
            {...formItemLayoutWithOutLabel}
            label={"Сокращение: "}
            required={false}
            key="termAbbreviation{term.id}"
          >
            {getFieldDecorator(`abbreviation`, {
              initialValue: term.abbreviation
            })(
              <Input
                placeholder="Сокращение"
                style={{ width: "80%", marginRight: 8 }}
              />
            )}
          </Form.Item>

          <Form.Item
            {...formItemLayoutWithOutLabel}
            label={"Англ.: "}
            required={false}
            key="termEnName{term.id}"
          >
            {getFieldDecorator(`enName`, { initialValue: term.enName })(
              <Input
                placeholder="На английском"
                style={{ width: "80%", marginRight: 8 }}
              />
            )}
          </Form.Item>

          {formItems}

          <Form.Item>
            <Button type="dashed" onClick={this.add} style={{ width: "40%" }}>
              <Icon type="plus" />
              Добавить определение
            </Button>
            <Button type="small" htmlType="submit">
              Сохранить
            </Button>
            ,
            <Button type="small" onClick={this.handleCancel}>
              Отменить
            </Button>
            ,{delButton}
          </Form.Item>
        </Form>
      </Modal>
    );
  }
}
export const EditForm = Form.create({ name: "EditFormItem" })(EditFormClass);