import React from "react";
import { Tree } from "antd";
import "./App.css";
import axios from "axios";
import broker from "./Broker.js";

//Дерево выбора терминов

const { TreeNode } = Tree;

export class TermSelectTree extends React.Component {
  state = { treeData: [] };

  constructor(props) {
    super(props);
    broker.onEvent("searchClick", data => this.onSearchClick(data));
  }

  onSearchClick(value) {
    //todo обновление дерева при поиске из строки
    //пока не получилось - надо найти способ полной переинициализации компонента,
    //просто перезагрузка нод дерева+сброс всяких *Keys не помогает
  }

  onSelect = (selectedKeys, value) => {
    //показываем выбранный термин
    if (value.node.props.isLeaf) {
      broker.storeEvent("termSelected", value.node.props.dataRef.key);
    }
    this.setState({ selectedKeys });
  };

  componentDidMount() {
    //заполняем верхний уровень дерева первыми букавами наименований терминов
    axios.get(`/getTermsNamesFls`).then(res => {
      this.setState({ treeData: [] });
      const data = res.data;
      data.forEach(item => {
        const trdata = this.state.treeData.concat({
          title: item.fl,
          key: item.fl
        });
        this.setState({ treeData: [...trdata] });
      });
    });
  }

  onLoadData = treeNode =>
    // при развертывании ветки дерева на выбранную букву
    new Promise(resolve => {
      if (treeNode.props.children) {
        resolve();
        return;
      }
      treeNode.props.dataRef.children = [];
      axios.get(`/getTermsNamesByFl/${treeNode.props.eventKey}`).then(res => {
        const data = res.data;
        data.forEach(item => {
          treeNode.props.dataRef.children.push({
            title: item.name,
            key: item.id,
            isLeaf: true
          });
        });
        this.setState({ treeData: [...this.state.treeData] });
        resolve();
      });
    });

  renderTreeNodes = data =>
    data.map(item => {
      if (item.children) {
        return (
          <TreeNode title={item.title} key={item.key} dataRef={item}>
            {this.renderTreeNodes(item.children)}
          </TreeNode>
        );
      }
      return <TreeNode key={item.key} {...item} dataRef={item} />;
    });

  render() {
    return (
      <Tree onSelect={this.onSelect} loadData={this.onLoadData}>
        {this.renderTreeNodes(this.state.treeData)}
      </Tree>
    );
  }
}
