const mysql = require(`mysql`);

const basicQuery = `SELECT terms.id as id,terms.name as name,terms.abbreviation as abbreviation, terms.en_name as en_name, descriptions.id as description_id,descriptions.text as description_text, descriptions.source_id as description_source_id FROM terms left join descriptions on terms.id=descriptions.term_id`;

var sql = mysql.createPool({
  host: `localhost`,
  user: `User`,
  password: `Password`,
  database: `termsdb`
});

//общая функция для получения терминов по тексту SQL-запроса
getTermsbySQL = (sqlText, result) => {
  sql.query(sqlText, (err, res) => {
    if (err) {
      result(err, null);
      return;
    }

    //в любом случае возвращаем массив
    const terms = [];
    if (res.length) {
      //возвращаем ответ в структурированном виде
      let tempDescriptions = [];
      let currentTermId = -1;
      for (let i = 0; i < res.length; i++) {
        if (currentTermId != res[i].id) {
          if (currentTermId != -1) {
            terms[terms.length - 1].descriptions = tempDescriptions;
            tempDescriptions = [];
          }
          terms.push({
            id: res[i].id,
            name: res[i].name,
            abbreviation: res[i].abbreviation,
            enName: res[i].en_name,
            descriptions: []
          });
          currentTermId = res[i].id;
        }
        tempDescriptions.push({
          id: res[i].description_id,
          text: res[i].description_text,
          sourceId: res[i].description_source_id
        });
      }
      if (currentTermId != -1) {
        terms[terms.length - 1].descriptions = tempDescriptions;
        tempDescriptions = [];
      }
    }
    result(null, terms);
    return;
  });
};

const term = () => {};

//возвращает термин по его id
term.getTerm = (termId, result) => {
  getTermsbySQL(
    basicQuery +
      ` WHERE terms.id = ${termId} order by terms.id, descriptions.id`,
    (err, res) => {
      if (err) {
        result(err, null);
        return;
      }
      result(null, res);
      return;
    }
  );
};

//возвращает набор терминов вхождению фразы searchString в название
term.getTermsbySearchString = (searchString, result) => {
  getTermsbySQL(
    basicQuery +
      ` WHERE lower(terms.name) like lower('%${searchString}%') order by terms.id, descriptions.id`,
    (err, res) => {
      if (err) {
        result(err, null);
        return;
      }
      result(null, res);
      return;
    }
  );
};

//создание или изменение термина
term.storeTerm = (term, result) => {
  if (term.id > -1) {
    //редактируем существующий
    sql.query(
      `UPDATE terms SET name = '${term.name}', abbreviation = '${term.abbreviation}', en_name = '${term.enName}' WHERE id = ${term.id};`,
      (err, res) => {
        if (err) throw err;
        //вот тут остроумно, конечно - не обновляем определения, а просто все старые удаляем и пишем новые
        sql.query(
          `DELETE FROM descriptions WHERE term_id = ${term.id}`,
          (err, res) => {
            //новые вставляем только после удаления предыдущих
            //и ждем, пока вставятся все
            let insertResults = [];
            for (let d = 0; d < term.descriptions.length; d++) {
              let promise = new Promise(resolve => {
                sql.query(
                  `INSERT INTO descriptions (term_id,text,source_id) VALUES ('${term.id}','${term.descriptions[d].text}','${term.descriptions[d].sourceId}');`,
                  (err, res) => {
                    resolve();
                  }
                );
              });
              insertResults.push(promise);
            }

            Promise.all(insertResults).then(values => {
              result(null, { id: term.id });
              return;
            });
          }
        );
      }
    );
  } else {
    //создаем новый
    sql.query(
      `INSERT INTO terms (name,abbreviation,en_name) VALUES ('${term.name}','${term.abbreviation}','${term.enName}');`,
      (err, res) => {
        if (err) throw err;

        //и ждем, пока вставятся все
        let insertResults = [];
        for (let d = 0; d < term.descriptions.length; d++) {
          let promise = new Promise(resolve => {
            sql.query(
              `INSERT INTO descriptions (term_id,text,source_id) VALUES ('${res.insertId}','${term.descriptions[d].text}','${term.descriptions[d].sourceId}');`,
              (err, res) => {
                resolve();
              }
            );
          });
          insertResults.push(promise);
        }

        Promise.all(insertResults).then(values => {
          result(null, { id: res.insertId });
          return;
        });
      }
    );
  }
};

//удаляем термин и все его определения
term.deleteTerm = (termId, result) => {
  sql.query(`DELETE FROM terms WHERE id = ${termId}`, (err, res) => {
    sql.query(
      `DELETE FROM descriptions WHERE term_id = ${termId}`,
      (err, res) => {
        result(null, { id: -1 });
        return;
      }
    );
  });
};

//возвращает набор первых букв названий всех терминов
term.getTermsNamesFls = result => {
  sql.query(
    `SELECT DISTINCT left(name,1) as fl FROM terms order by name`,
    (err, res) => {
      if (err) {
        result(err, null);
        return;
      }
      result(null, res);
      return;
    }
  );
};

//возвращает набор терминов начинающихся на определенную букву
term.getTermsNamesByFl = (termFl, result) => {
  sql.query(
    `SELECT id,name FROM terms WHERE left(name,1) like '${termFl}'`,
    (err, res) => {
      if (err) {
        result(err, null);
        return;
      }
      result(null, res);
      return;
    }
  );
};

//получаем список для autocomplite
term.getTermsNamesforAC = (searchString, result) => {
  sql.query(
    `SELECT name FROM terms WHERE lower(name) like lower('${searchString}%') order by name limit 3`,
    (err, res) => {
      if (err) {
        result(err, null);
        return;
      }
      result(null, res);
      return;
    }
  );
};

//получение справочника источников описаний терминов
term.getSources = result => {
  sql.query(
    `SELECT sources.id as id, sources.name as name, sources.doc_name as doc_name,src_kinds.name as src_kid_name FROM sources join src_kinds on sources.src_kind_id=src_kinds.id order by sources.id`,
    (err, res) => {
      if (err) {
        result(err, null);
        return;
      }
      //возвращаем ответ c корректными наименованиями свойств, а не как в БД
      //можно конечно просто as поменять в запросе, но так нагляднее
      const sources = [];
      for (let i = 0; i < res.length; i++) {
        sources.push({
          id: res[i].id,
          name: res[i].name,
          docName: res[i].doc_name,
          srcKidName: res[i].src_kid_name
        });
      }
      result(null, sources);
      return;
    }
  );
};

module.exports = term;
